#include <ggi/ggiart.h>
#include <time.h>

#define MIN(a,b) (a<b?a:b)

ggi_visual_t vis;

ggi_color colh = { 0xffff, 0x0, 0x0, 0x8f8f };
ggi_color colm = { 0x0, 0xdfff, 0x0, 0x8f8f };
ggi_color cols = { 0x0, 0x0, 0xffff, 0x8f8f };

ggi_color bgcolor = { 0xdfdf, 0xdfdf, 0xffff, 0xffff };
ggi_color bordercolor = { 0xffff, 0x8ff0, 0xeff0, 0x6f00 };

ggi_pixel bgpixel;

struct ggiart_rop_s rh;
struct ggiart_rop_s rm;
struct ggiart_rop_s rs;
struct ggiart_rop_s rborder;

ArtVpath   vh[5];
ArtVpath   vm[5];
ArtVpath   vs[5];

ggiart_svp_t border;

double translate[6]; /* center */

inline void render(ArtVpath   v[5], struct ggiart_rop_s *rop, double angle) {
  ggiart_svp_t svp;
  double transform[6];
  ArtVpath   *path;

  art_affine_rotate (transform,angle);
  art_affine_multiply (transform,transform,translate);
  path=art_vpath_affine_transform(v,transform);
  svp = art_svp_from_vpath(path);

  ggiartRenderSVP(vis,svp,NULL,rop);
  art_svp_free(svp);

  art_free(path);
}

void update(void) {
  time_t ct;
  struct tm *lt;

  ggiFillscreen(vis);

  time(&ct);
  lt=localtime(&ct);

  //  ggiartRenderSVP(vis,border,NULL,&rborder);

  render(vh,&rh,((lt->tm_hour%12)*((double)30.0))+(lt->tm_min%60)/((double)2.0));
  render(vm,&rm,((lt->tm_min%60)*((double)6.0))+(lt->tm_sec%60)/((double)10.0));
  render(vs,&rs,(lt->tm_sec%60)*((double)6.0));

  ggiFlush(vis);

}

static build(ArtVpath p[5],double h, double w,double c) {
   
  p[0].code=ART_MOVETO;
  p[0].x=w/2.0;
  p[0].y=c;

  p[1].code=ART_LINETO;
  p[1].x=0.0;
  p[1].y=c-h;

  p[2].code=ART_LINETO;
  p[2].x=-p[0].x;
  p[2].y=p[0].y;

  p[3].code=ART_LINETO;
  p[3].x=p[0].x;
  p[3].y=p[0].y;
  p[4].code=ART_END;
}

void setup(void) {
  
  ArtVpath   *path;

  ggi_mode mode;
  double r;

  rh.rop=GGIART_ROP_COLOR;
  rm.rop=GGIART_ROP_COLOR;
  rs.rop=GGIART_ROP_COLOR;
  rborder.rop=GGIART_ROP_COLOR;
  

  rh.arg1=(void*)&colh;
  rm.arg1=(void*)&colm;
  rs.arg1=(void*)&cols;
  rborder.arg1=(void*)&bordercolor;
  

  bgpixel=ggiMapColor(vis,&bgcolor);
  ggiSetGCForeground(vis,bgpixel);

  ggiGetMode(vis,&mode);
  
  art_affine_translate(translate,mode.visible.x/2.0, mode.visible.y/2.0);
  
  r=MIN(mode.visible.x/2.0, mode.visible.y/2.0);

  build(vh,r/2.0,r/10.0,-r/50.0);
  build(vm,r/1.3,r/15.0,-r/50.0);
  build(vs,r/1.1,r/20.0,-r/50.0);

  path=art_vpath_new_circle(mode.visible.x/2.0, mode.visible.y/2.0,r-r/50.0);
  border=art_svp_vpath_stroke(path,
			      ART_PATH_STROKE_JOIN_MITER,
			      ART_PATH_STROKE_CAP_SQUARE,r/100.0,4,7);

}

int get_visual(void) {
  ggi_mode mode;
  if((vis=ggiOpen(NULL))==NULL) {
    fprintf(stderr,"Couldn't open visual.\n");
    return -1;
  }
  ggiCheckGraphMode(vis,GGI_AUTO,GGI_AUTO,GGI_AUTO,GGI_AUTO,GGI_AUTO,&mode);
  ggiSetMode(vis,&mode);
  ggiSetEventMask(vis,emKeyPress);
  ggiartAttach(vis);
  ggiAddFlags(vis, GGIFLAG_ASYNC);
  return 0;
}

int main() {
  struct timeval t;
  gii_event ev;
  gii_event_mask em;
  if (ggiInit()<0) {
    fprintf(stderr,"Couldn't init ggi.\n");
    return -1;
  }
  
  if (ggiartInit()<0) {
    fprintf(stderr,"Couldn't init ggiart.\n");
    ggiExit();
    return -1;
  }


  if (get_visual()<0) {
    ggiartExit();
    ggiExit();
    return -1;
  }

  setup();
  update();
  while(1) {
    t.tv_sec=1;
    t.tv_usec=0;
    em=ggiEventPoll(vis,emKeyPress, &t);
    if(em==emKeyPress) {
      break;
    }
    update();
  }
  ggiartDetach(vis);
  ggiartExit();
  ggiClose(vis);
  ggiExit();
  return 0;
}
