/*
******************************************************************************

   GGIART debugging macros

   Copyright (C) 2000 Eric Faurot       	[eric@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GGIART_INTERNAL_DEBUG_H
#define _GGIART_INTERNAL_DEBUG_H

#include <stdio.h>
#include <stdarg.h>
#include <ggi/types.h>
#include <ggi/gg.h>

#include <ggi/ggiart.h>


__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_GGIART
extern uint32     _ggiartDebugState;
extern int        _ggiartDebugSync;
#else
IMPORTVAR uint32  _ggiartDebugState;
IMPORTVAR int     _ggiartDebugSync;
#endif
__END_DECLS


/* Debugging types
 * bit 0 is reserved! */

#define GGIARTDEBUG_CORE		(1<<1)	/*   2 */
#define GGIARTDEBUG_MODE		(1<<2)	/*   4 */
#define GGIARTDEBUG_COLOR		(1<<3)	/*   8 */
#define GGIARTDEBUG_DRAW		(1<<4)	/*  16 */
#define GGIARTDEBUG_MISC		(1<<5)	/*  32 */
#define GGIARTDEBUG_LIBS		(1<<6)	/*  64 */
#define GGIARTDEBUG_EVENTS		(1<<7)	/* 128 */

#define GGIARTDEBUG_ALL	0xffffffff

#ifdef __GNUC__

#ifdef DEBUG
#define GGIARTDPRINT(form,args...)	   if (_ggiartDebugState) { ggDPrintf(_ggiartDebugSync, "LibGGIART",form, ##args); }
#define GGIARTDPRINT_CORE(form,args...)   if (_ggiartDebugState & GGIARTDEBUG_CORE) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_MODE(form,args...)   if (_ggiartDebugState & GGIARTDEBUG_MODE) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_COLOR(form,args...)  if (_ggiartDebugState & GGIARTDEBUG_COLOR) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_DRAW(form,args...)   if (_ggiartDebugState & GGIARTDEBUG_DRAW) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_MISC(form,args...)   if (_ggiartDebugState & GGIARTDEBUG_MISC) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_LIBS(form,args...)   if (_ggiartDebugState & GGIARTDEBUG_LIBS) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#define GGIARTDPRINT_EVENTS(form,args...) if (_ggiartDebugState & GGIARTDEBUG_EVENTS) { ggDPrintf(_ggiartDebugSync,"LibGGIART",form, ##args); }
#else /* DEBUG */
#define GGIARTDPRINT(form,args...)		do{}while(0)
#define GGIARTDPRINT_CORE(form,args...)	do{}while(0)
#define GGIARTDPRINT_MODE(form,args...)	do{}while(0)
#define GGIARTDPRINT_COLOR(form,args...)	do{}while(0)
#define GGIARTDPRINT_DRAW(form,args...)	do{}while(0)
#define GGIARTDPRINT_MISC(form,args...)	do{}while(0)
#define GGIARTDPRINT_LIBS(form,args...)	do{}while(0)
#define GGIARTDPRINT_EVENTS(form,args...)	do{}while(0)
#endif /* DEBUG */

#else /* __GNUC__ */

__BEGIN_DECLS

static inline void GGIARTDPRINT(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_CORE(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_CORE) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_MODE(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_MODE) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_COLOR(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_COLOR) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_DRAW(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_DRAW) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_MISC(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_MISC) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_LIBS(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_LIBS) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

static inline void GGIARTDPRINT_EVENTS(const char *form,...)
{
#ifdef DEBUG
	if (_ggiDebugState & GGIDEBUG_EVENTS) {
		va_list args;

		fprintf(stderr, "GGIART: ");
		va_start(args, form);
		vfprintf(stderr, form, args);
		va_end(args);
		if (_ggiartDebugSync) fflush(stderr);
	}
#endif
}

__END_DECLS

#endif /* __GNUC__ */

#ifdef DEBUG
#define GGIART_ASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"GGIART:%s:%d: INTERNAL ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#define GGIART_APPASSERT(x,str) \
{ if (!(x)) { \
	fprintf(stderr,"GGIART:%s:%d: APPLICATION ERROR: %s\n",__FILE__,__LINE__,str); \
	exit(1); \
} }
#else /* DEBUG */
#define GGIART_ASSERT(x,str)	do{}while(0)
#define GGIART_APPASSERT(x,str)	do{}while(0)
#endif /* DEBUG */

#ifdef DEBUG
# define GGIARTD0(x)	x
#else
# define GGIARTD0(x)	/* empty */
#endif

#ifdef GGIARTDLEV
# if GGIARTDLEV == 1
#  define GGIARTD1(x)	x
#  define GGIARTD2(x)	/* empty */
#  define GGIARTD3(x)	/* empty */
# elif GGIARTDLEV == 2
#  define GGIARTD1(x)	x
#  define GGIARTD2(x)	x
#  define GGIARTD3(x)	/* empty */
# elif GGIARTDLEV > 2
#  define GGIARTD1(x)	x
#  define GGIARTD2(x)	x
#  define GGIARTD3(x)	x
# endif
#else
# define GGIARTD1(x)	/* empty */
# define GGIARTD2(x)	/* empty */
# define GGIARTD3(x)	/* empty */
#endif

#endif /* _GGIART_INTERNAL_DEBUG_H */
